from flask import render_template, Blueprint, request, flash
from sqlalchemy.orm import Query

from src.classes.base import RearrangeDate
from src.classes.family import AddFamily, EditFamily
from src.classes.database import Child, Family, sessionSetup, Parent
from src.modules.children import find_related_children
from src.modules.common import stringdate, find_parents, generate_uuid, find_active_class
from src.modules.families import find_family, find_families, provision_edit_family, find_all_parents

session = sessionSetup()
rearrange_date = RearrangeDate()

# Blueprint Configuration
families_bp = Blueprint(
    'families_bp', __name__,
    template_folder='templates'
)


@families_bp.route('/index_families')
def index_families():
    _families = find_families(uuid=None)
    return render_template('index_families.html',
                           Families=_families,
                           _PageTitle='Familie Overzicht')


@families_bp.route('/details_family/<uuid>')
def details_family(uuid):
    _family = provision_edit_family(uuid)
    _children = find_related_children(uuid)
    print(_children)

    return render_template('details_family.html',
                           Family=_family,
                           Children=_children,
                           _PageTitle='Gezins details')


@families_bp.route('/edit_family/<uuid>')
def edit_family(uuid):
    _familyToEdit = provision_edit_family(uuid)
    print(_familyToEdit)
    _form1 = EditFamily()
    _form1.parent1_uuid.choices = [(_parent['uuid'], _parent['fullname'])
                                   for _parent in find_all_parents()]
    _form1.parent2_uuid.choices = [(_parent['uuid'], _parent['fullname'])
                                   for _parent in find_all_parents()]
    return render_template('edit_family.html',
                           familyToEdit=_familyToEdit,
                           form1=_form1,
                           _PageTitle='Gezin wijzigen')


@families_bp.route('/add_family', methods=['GET', 'POST'])
def add_family():
    _form1 = AddFamily()
    # _form1.parent1_id.choices = Query([Parent.id, Parent.firstname + ' ' + Parent.lastname]).with_session(session)
    _form1.parent1_uuid.choices = [(_parent['uuid'], _parent['fullname'])
                                   for _parent in find_all_parents()]
    _form1.parent1_uuid.choices.insert(0, (None, 'Kies een ouder'))
    _form1.parent2_uuid.choices = [(_parent['uuid'], _parent['fullname'])
                                   for _parent in find_all_parents()]
    _form1.parent2_uuid.choices.insert(0, ('x-x-x-x', 'Kies een ouder'))
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        parent1_uuid = request.form['parent1_uuid']
        parent2_uuid = request.form['parent2_uuid']
        divorced = request.form['divorced']

        if request.form['parent2_uuid'] is 'x-x-x-x':
            parent2_uuid = None
        # the data to be inserted into Sock model - the table, socks
        record = Family(id_field, parent1_uuid, parent2_uuid, divorced, generate_uuid())
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        session.close()
        # create a message to send to the template
        _message = f"Gezin {id_field} is aangemaakt."
        return render_template('add_family.html',
                               message=_message,
                               _PageTitle='Gezin toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_family.html',
                               form1=_form1,
                               _PageTitle='Gezin toevoegen')


@families_bp.route('/remove_family/<uuid>')
def remove_family(uuid):
    try:
        _child_to_delete = Query(Child).with_session(session).filter(Child.family_uuid == uuid).delete()
    except:
        pass
    _family_to_delete = Query(Family).with_session(session).filter(Family.uuid == uuid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_family_result.html',
                           message=message,
                           _PageTitle='Gezin verwijderen')


@families_bp.route('/edit_family_result', methods=['POST'])
def edit_family_result():
    uuid = request.form['uuid_field']
    _familyToEdit = provision_edit_family(uuid)
    _form1 = EditFamily()
    _family_to_edit = Query(Family).with_session(session).filter(Family.uuid == uuid).update(dict(
        parent1_uuid=_form1.parent1_uuid.data,
        parent2_uuid=_form1.parent2_uuid.data,
        divorced=_form1.divorced.data))
    session.commit()
    message = f"De gegevens voor {_familyToEdit['parent1']['lastname']} zijn bijgewerkt."
    return render_template('result.html',
                           message=message,
                           redirect=f'details_family/{uuid}',
                           _PageTitle='Resultaat')




from datetime import date

from sqlalchemy.orm import Query

from src.classes.child import rearrange_date
from src.classes.database import Child, sessionSetup, Teacher, Class, Family, Parent
from src.modules.classrooms import get_classroom
from src.modules.common import find_parents, calculate_age, find_class

session = sessionSetup()


def find_related_children(uuid):
    query_entities = [
        Child.uuid,
        Child.family_uuid,
        Child.firstname,
        Child.lastname,
        Child.date_of_birth,
        Child.class1_uuid,
        Child.class2_uuid,
        Child.class3_uuid
    ]
    _queried_child = Query(query_entities).filter(Child.family_uuid == uuid).with_session(session)
    children = []
    for c in _queried_child:
        children.append({
            'uuid': c['uuid'],
            'family_uuid': c['family_uuid'],
            'firstname': c['firstname'],
            'lastname': c['lastname'],
            'age': calculate_age(c['date_of_birth']),
            'classroom1': get_classroom(c['class1_uuid']),
            'classroom2': get_classroom(c['class2_uuid']),
            'classroom3': get_classroom(c['class3_uuid'])
        })
    return children



from datetime import date

from sqlalchemy.orm import Query

from src.classes.database import Child, sessionSetup, Teacher, Class, Family, Parent
from src.modules.common import find_parents

session = sessionSetup()


def find_all_parents():
    # session.close()
    _parents = Query(Parent).with_entities(Parent.uuid, Parent.firstname, Parent.lastname).with_session(session)
    _all_parents = []
    for _parent in _parents:
        _all_parents.append({
            'uuid': _parent['uuid'],
            'fullname': _parent['firstname'] + ' ' + _parent['lastname'],
        })
    # session.close()
    return _all_parents


def find_families(uuid):
    query_entities = [
        Family.uuid,
        Family.parent1_uuid,
        Family.parent2_uuid,
        Family.divorced,
    ]
    if uuid is None:
        _families_from_database = Query(query_entities).with_session(session)
    else:
        _families_from_database = Query(query_entities).filter(Family.uuid == uuid).with_session(session)
    _families = []
    for _family in _families_from_database:
        _families.append({
            'uuid': _family['uuid'],
            'parent1_uuid': _family['parent1_uuid'],
            'parent1': find_parents(_family['parent1_uuid']),
            'parent2_uuid': _family['parent2_uuid'],
            'parent2': find_parents(_family['parent2_uuid']),
            'divorced': _family['divorced']
        })
    return _families


def provision_edit_family(uuid):
    _family = find_families(uuid)[0]
    _parent1 = find_parents(_family['parent1_uuid'])
    _parent2 = find_parents(_family['parent2_uuid'])
    _family_to_edit = {'uuid': _family['uuid'],
                       'divorced': _family['divorced'],
                       'parent1': {
                           'uuid': _parent1['uuid'],
                           'firstname': _parent1['firstname'],
                           'lastname': _parent1['lastname'],
                           'address': _parent1['address'],
                           'city': _parent1['city'],
                           'zipcode': _parent1['zipcode'],
                           'phone': _parent1['phone'],
                           'email': _parent1['email']
                       },
                       }
    if _parent2:
        _family_to_edit.update({
            'parent2': {
                'uuid': _parent2['uuid'],
                'firstname': _parent2['firstname'],
                'lastname': _parent2['lastname'],
                'address': _parent2['address'],
                'city': _parent2['city'],
                'zipcode': _parent2['zipcode'],
                'phone': _parent2['phone'],
                'email': _parent2['email']
            }})
    else:
        _family_to_edit.update({
            'parent2': {}})
    return _family_to_edit


def find_family():
    _families_from_database = find_families('')
    # family table
    # parents opzoeken
    # tuple aanmaken voor ouders met namen enzo



from flask_wtf import FlaskForm
from sqlalchemy.orm import Query
from wtforms import SubmitField, HiddenField, StringField, BooleanField, SelectField
from wtforms.validators import InputRequired, Length, Regexp
from flask_sqlalchemy import SQLAlchemy
from src.modules.common import calculate_age, generate_uuid
from src.classes.database import Child, sessionSetup, Class, Parent

db = SQLAlchemy()
session = sessionSetup()


class AddFamily(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    parent1_uuid = SelectField(u'Ouder1', [InputRequired()], coerce=str)
    parent2_uuid = SelectField(u'Ouder2', coerce=str)
    divorced = SelectField(u'Gescheiden',
                           coerce=int,
                           choices=[(0, 'Nee'), (1, 'Ja')])
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('Add/Update Record')


class EditFamily(FlaskForm):
    # id used only by update/edit
    uuid_field = HiddenField()
    parent1_uuid = SelectField(u'Ouders', [InputRequired()],
                             coerce=str)
    parent2_uuid = SelectField(u'Ouders',
                             coerce=str)
    divorced = SelectField(u'Gescheiden',
                           coerce=int,
                           choices=[(0, 'Nee'), (1, 'Ja')])
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('opslaan')




from importlib import reload

from flask import render_template, Blueprint, request, flash
from sqlalchemy.orm import Query
from src.modules.common import stringdate, calculate_age, calculate_startyear, find_class, find_teacher, \
    query_class_by_year, find_child, get_appropriate_class, get_family, generate_uuid
from src.classes.base import RearrangeDate
from src.classes.child import AddChild, EditChild
from src.classes.database import Child, Family, sessionSetup
from src.modules.families import provision_edit_family

session = sessionSetup()
rearrange_date = RearrangeDate()

# Blueprint Configuration
children_bp = Blueprint(
    'children_bp', __name__,
    template_folder='templates'
)


@children_bp.route('/index_children')
def index_children():
    # get a list of unique values in the style column
    _children_from_database = Query([
        Child.firstname,
        Child.lastname,
        Child.date_of_registration,
        Child.date_of_birth,
        Child.uuid
    ]).with_session(session).order_by(
        Child.date_of_birth.asc(),
        Child.date_of_registration.asc()
    )
    _children = []
    for _child in _children_from_database:
        _children.append({
            'firstname': _child[0],
            'lastname': _child[1],
            'date_of_registration': rearrange_date.to_list(_child[2])[0],
            'date_of_birth': rearrange_date.to_list(_child[3])[0],
            'uuid': _child[4],
            '_starts_in': calculate_startyear(date_of_birth=_child[3])
        })
    return render_template('index_children.html',
                           Child=_children,
                           _PageTitle=f'Kinder overzicht')


@children_bp.route('/details_child/<uuid>')
def details_child(uuid):
    _child_from_database = find_child(uuid)
    _family = provision_edit_family(_child_from_database['family_uuid'])
    _child = []
    if _child_from_database['class1_uuid'] is None or "None":
        _class_name = None
        _class_teacher = None
    else:
        _class_name = find_class(_child_from_database['class1_uuid'])
        _class_teacher = find_teacher(_class_name[1])
    _child.append({
        'firstname': _child_from_database['firstname'],
        'lastname': _child_from_database['lastname'],
        'date_of_registration': rearrange_date.to_list(_child_from_database['date_of_registration'])[0],
        'date_of_birth': rearrange_date.to_list(_child_from_database['date_of_birth'])[0],
        'uuid': _child_from_database['uuid'],
        'class_name': _class_name,
        'class_teacher': _class_teacher
    })
    _child_age = calculate_age(_child_from_database['date_of_birth'])
    return render_template('details_child.html',
                           Child=_child,
                           ChildAge=_child_age,
                           Parents=_family,
                           _PageTitle='Kind details')


@children_bp.route('/add_child/', methods=['GET', 'POST'])
def add_child():
    _form1 = AddChild()
    _form1.family_uuid.choices = [(_fam['family_uuid'], _fam['parent1'] + ' & ' + _fam['parent2'])
                                  for _fam in get_family()]
    _form1.family_uuid.choices.insert(0, (0, 'Kies een gezin'))
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        date_of_registration = request.form['date_of_registration']
        date_of_registration = rearrange_date.to_order(date_of_registration)[0]
        family_uuid = request.form['family_uuid']
        date_of_birth = request.form['date_of_birth']
        date_of_birth = rearrange_date.to_order(date_of_birth)[0]
        class1_uuid = 'None'
        class2_uuid = 'None'
        class3_uuid = 'None'
        # the data to be inserted into Sock model - the table, socks
        record = Child(id_field, firstname, lastname, date_of_registration, family_uuid, date_of_birth,
                       class1_uuid, class2_uuid, class3_uuid, generate_uuid())
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"Het kind {firstname} {lastname} is aangemaakt."
        return render_template('add_child.html',
                               message=_message,
                               _PageTitle='Kind toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_child.html',
                               form1=_form1,
                               _PageTitle='Kind toevoegen')


@children_bp.route('/edit_child/<uuid>')
# TODO
# Klas toewijzen?
def edit_child(uuid):
    _child_from_database = Query([
        Child.firstname,
        Child.lastname,
        Child.date_of_birth,
        Child.date_of_registration,
        Child.uuid,
        Child.family_uuid,
        Child.class1_uuid,
        Child.class2_uuid,
        Child.class3_uuid
    ]).with_session(session).filter(Child.uuid == uuid).first()
    print(f'FamilyID: {_child_from_database.family_uuid}')
    print(f'all: {_child_from_database}')
    # two forms in this template
    _child_to_edit = {'firstname': _child_from_database.firstname,
                      'lastname': _child_from_database.lastname,
                      'date_of_birth': rearrange_date.to_list(_child_from_database.date_of_birth)[0],
                      'date_of_registration': rearrange_date.to_list(_child_from_database.date_of_registration)[0],
                      'uuid': _child_from_database.uuid,
                      'family_uuid': _child_from_database.family_uuid,
                      'class1_uuid': _child_from_database.class1_uuid,
                      'class2_uuid': _child_from_database.class2_uuid,
                      'class3_uuid': _child_from_database.class3_uuid}
    _form1 = EditChild()
    _form1.class1_uuid.choices = get_appropriate_class(uuid, years=4)
    _form1.class2_uuid.choices = get_appropriate_class(uuid, years=4)
    _form1.class3_uuid.choices = get_appropriate_class(uuid=uuid, years=6)
    _form1.family_uuid.choices = [(_fam['family_uuid'], _fam['parent1'] + ' & ' + _fam['parent2'])
                                  for _fam in get_family()]

    return render_template('edit_child.html',
                           childToEdit=_child_to_edit,
                           form1=_form1,
                           _PageTitle=f'Kind wijzigen')


@children_bp.route('/remove_child/<uuid>')
def remove_child(uuid):
    _child_to_remove = Query(Child).with_session(session).filter(Child.uuid == uuid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_child_result.html',
                           message=message,
                           _PageTitle='Kind verwijderen')


@children_bp.route('/edit_child_result', methods=['POST'])
def edit_child_result():
    uuid = request.form['uuid_field']
    _child_parents = Query(Child.family_uuid).filter(Child.uuid == uuid).with_session(session).first()[0]
    _childToEdit = {"uuid": uuid,
                    "firstname": request.form['firstname'],
                    "lastname": request.form['lastname'],
                    "date_of_birth": rearrange_date.to_order(request.form['date_of_birth'])[0],
                    "date_of_registration": rearrange_date.to_order(request.form['date_of_registration'])[0],
                    "family_uuid": request.form['family_uuid'],
                    "class1_uuid": request.form['class1_uuid'],
                    "class2_uuid": request.form['class2_uuid'],
                    "class3_uuid": request.form['class3_uuid'],
                    "updated": stringdate()}

    available_groups = query_class_by_year(
        calculate_startyear(Query(Child.date_of_birth).filter_by(uuid=uuid).with_session(session)[0][0])[3:7])
    # Now forming the list of tuples for SelectField
    _form1 = EditChild()
    _form1.class1_uuid.choices = get_appropriate_class(uuid=uuid, years=4)
    _form1.class2_uuid.choices = get_appropriate_class(uuid=uuid, years=4)
    _form1.class3_uuid.choices = get_appropriate_class(uuid=uuid, years=6)
    _form1.family_uuid.choices = [(_fam['family_uuid'], _fam['parent1'] + ' & ' + _fam['parent2'])
                                  for _fam in get_family()]
    _child_to_edit = Query(Child).with_session(session).filter(Child.uuid == uuid).update(dict(
        firstname=_form1.firstname.data,
        lastname=_form1.lastname.data,
        date_of_registration=_form1.date_of_registration.data,
        date_of_birth=_form1.date_of_birth.data,
        class1_uuid=_form1.class1_uuid.data,
        class2_uuid=_form1.class2_uuid.data,
        class3_uuid=_form1.class3_uuid.data
    ))
    session.commit()
    message = f"De gegevens voor {_childToEdit['firstname']} zijn bijgewerkt."
    return render_template('result.html',
                           message=message,
                           redirect=f'details_child/{uuid}',
                           _PageTitle='Resultaat')



from flask import render_template, Blueprint, request, flash
from sqlalchemy.orm import Query

from src.classes.base import RearrangeDate
from src.classes.family import AddFamily, EditFamily
from src.classes.database import Child, Family, sessionSetup, Parent
from src.classes.parent import EditParent, AddParent
from src.modules.common import stringdate, find_children, generate_uuid

session = sessionSetup()
rearrange_date = RearrangeDate()


# Blueprint Configuration
parents_bp = Blueprint(
    'parents_bp', __name__,
    template_folder='templates'
)


@parents_bp.route('/index_parents')
def index_parents():
    # get a list of unique values in the style column
    _parents_from_database = Query([
        Parent.uuid,
        Parent.lastname,
        Parent.firstname,
        Parent.address,
        Parent.zipcode,
        Parent.city,
        Parent.email,
        Parent.phone
    ]).with_session(session).order_by(
        Parent.lastname.asc(),
    )
    _parents = []
    for _parent in _parents_from_database:
        _parents.append({
            'uuid': _parent[0],
            'lastname': _parent[1],
            'firstname': _parent[2],
            'address': _parent[3],
            'zipcode': _parent[4],
            'city': _parent[5],
            'email': _parent[6],
            'phone': _parent[7]
        })
    return render_template('index_parents.html',
                           Parents=_parents,
                           _PageTitle='Ouder Overzicht')


@parents_bp.route('/details_parent/<uuid>')
def details_parent(uuid):
    try:
        _parent_from_database = Query([
            Parent.firstname,
            Parent.lastname,
            Parent.address,
            Parent.zipcode,
            Parent.city,
            Parent.email,
            Parent.phone,
        ]).filter_by(uuid=uuid).with_session(session).first()
        _parent = {
                'uuid': uuid,
                'firstname': _parent_from_database.firstname,
                'lastname': _parent_from_database.lastname,
                'address': _parent_from_database.address,
                'zipcode': _parent_from_database.zipcode,
                'city': _parent_from_database.city,
                'email': _parent_from_database.email,
                'phone': _parent_from_database.phone
            }
        return render_template('details_parent.html',
                               Parent=_parent,
                               _PageTitle='Ouder details')
    except Exception as e:
        # e holds description of the error
        error_text = "<p>The error:<br>" + str(e) + "</p>"
        hed = '<h1>Something is broken.</h1>'
        return hed + error_text


@parents_bp.route('/edit_parent/<uuid>')
def edit_parent(uuid):
    _parent_from_database = Query([
        Parent.firstname,
        Parent.lastname,
        Parent.address,
        Parent.zipcode,
        Parent.city,
        Parent.email,
        Parent.phone,
        Parent.uuid,
    ]).filter_by(uuid=uuid).with_session(session).first()
    _parentToEdit = {
            'uuid': _parent_from_database.uuid,
            'firstname': _parent_from_database.firstname,
            'lastname': _parent_from_database.lastname,
            'address': _parent_from_database.address,
            'zipcode': _parent_from_database.zipcode,
            'city': _parent_from_database.city,
            'email': _parent_from_database.email,
            'phone': _parent_from_database.phone
        }
    _form1 = EditParent()
    return render_template('edit_parent.html',
                           parentToEdit=_parentToEdit,
                           form1=_form1,
                           _PageTitle='Ouder wijzigen')


@parents_bp.route('/add_parent', methods=['GET', 'POST'])
def add_parent():
    _form1 = AddParent()
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        address = request.form['address']
        zipcode = request.form['zipcode']
        city = request.form['city']
        email = request.form['email']
        phone = request.form['phone']
        # the data to be inserted into Sock model - the table, socks
        record = Parent(id_field, firstname, lastname, address, zipcode, city, email, phone, generate_uuid())
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"Ouder {firstname} {lastname} is aangemaakt."
        return render_template('add_parent.html',
                               message=_message,
                               _PageTitle='Ouder toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_parent.html',
                               form1=_form1,
                               _PageTitle='Ouder toevoegen')



@parents_bp.route('/remove_parent/<uuid>')
def remove_parent(uuid):
    _parent_to_delete = Query(Parent).with_session(session).filter(Parent.uuid == uuid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_parent_result.html',
                           message=message,
                           _PageTitle='Ouder verwijderen')


@parents_bp.route('/edit_parent_result', methods=['POST'])
def edit_parent_result():
    uuid = request.form['uuid_field']
    _parentToEdit = {"uuid": uuid,
                    "lastname": request.form['lastname'],
                    "firstname": request.form['firstname'],
                    "address": request.form['address'],
                    "zipcode": request.form['zipcode'],
                    "city": request.form['city'],
                    "email": request.form['email'],
                    "phone": request.form['phone'],
                    "updated": stringdate()}
    _form1 = EditParent()
    _parent_to_edit = Query(Parent).with_session(session).filter(Parent.uuid == uuid).update(dict(
        lastname=_form1.lastname.data,
        firstname=_form1.firstname.data,
        address=_form1.address.data,
        zipcode=_form1.zipcode.data,
        city=_form1.city.data,
        email=_form1.email.data,
        phone=_form1.phone.data))
    session.commit()
    message = f"De gegevens voor {_parentToEdit['lastname']} zijn bijgewerkt."
    return render_template('result.html',
                           message=message,
                           redirect=f'details_parent/{uuid}',
                           _PageTitle='Resultaat')



from datetime import date

from sqlalchemy.orm import Query, lazyload

from src.classes.base import RearrangeDate
from src.classes.database import Child, sessionSetup, Teacher, Class, Family, Parent
import uuid

session = sessionSetup()
rearrange_date = RearrangeDate()


def generate_uuid():
    _uuid = uuid.uuid4()
    return str(_uuid)


def find_children(uuid):
    _children_from_database = Query(
        [Child.id, Child.firstname, Child.date_of_birth, Child.family, Child.class_uuid]).filter(
        Child.family == uuid).with_session(session)
    _child = []
    for _c in _children_from_database:
        if _c[4] is None:
            _class_name = 'N/A'
        else:
            _class_name = Query(Class.class_name).filter(Class.id == _c[4]).with_session(session)[0][0]
        _child.append({
            'id': _c[0],
            'firstname': _c[1],
            'date_of_birth': _c[2],
            'child_age': calculate_age(_c[2]),
            'family': _c[3],
            'class': _class_name
        })
    return _child


def calculate_age(geboortedatum):
    from datetime import datetime, date

    _date_str = geboortedatum
    _dto = datetime.strptime(_date_str, '%Y-%m-%d').date()
    _today = date.today()
    calculated_age = _today.year - _dto.year - ((_today.month, _today.day) < (_dto.month, _dto.day))
    # self._leeftijd = age
    return calculated_age


def query_class_by_year(child_start_date):
    print(f'startdatum: {child_start_date}')
    classes_from_db = Query(Class).with_entities(Class.uuid, Class.start_date, Class.class_name, Class.teacher_uuid, Class.end_date).with_session(session).order_by(Class.start_date.asc())
    classes = []
    for c in classes_from_db:
        if c[1][0:4] <= child_start_date <= c[4][0:4]:
            print(f'in {c}')
            _c_teacher = Query(Teacher).with_entities(Teacher.uuid, Teacher.firstname, Teacher.lastname).with_session(session).filter_by(uuid=c[3])[0]
            print(f'cteacher: {_c_teacher}')
            classes.append({
                'class_uuid': c[0],
                'class_name': f"{c[2]} {c[1][0:4]} - {_c_teacher[1]} {_c_teacher[2]}"
            })
    return classes


def calculate_startyear(date_of_birth, years=4):
    from datetime import datetime
    from dateutil.relativedelta import relativedelta

    datetime_object = datetime.strptime(date_of_birth, "%Y-%m-%d").date()
    new_date = datetime_object + relativedelta(years=years)
    return datetime.strftime(new_date, "%m-%Y").replace(' 0', ' ')


def find_class(uuid):
    return Query(Class).with_entities(
        Class.class_name,
        Class.teacher_uuid,
        Class.start_date,
        Class.end_date,
        Class.uuid
    ).filter(Class.uuid == uuid).with_session(session)[0]


def find_parents(uuid):
    if uuid == 'x-x-x-x':
        return None
    return Query(Parent).with_entities(
        Parent.uuid,
        Parent.firstname,
        Parent.lastname,
        Parent.address,
        Parent.zipcode,
        Parent.city,
        Parent.phone,
        Parent.email
    ).filter(Parent.uuid == uuid).with_session(session).first()


def find_teacher(uuid):
    return Query(Teacher).with_entities(
        Teacher.firstname,
        Teacher.lastname,
        Teacher.uuid
    ).filter(Teacher.uuid == uuid).with_session(session)[0]


def find_all_teachers():
    _teachers = Query(Teacher).with_entities(Teacher.firstname, Teacher.lastname, Teacher.uuid).with_session(session)
    _all_teachers = []
    for _teacher in _teachers:
        _all_teachers.append({
            'uuid': _teacher['uuid'],
            'fullname': _teacher['firstname'] + ' ' + _teacher['lastname'],
        })
    return _all_teachers



def find_child(uuid):
    return Query(Child).with_entities(
        Child.firstname,
        Child.lastname,
        Child.date_of_registration,
        Child.date_of_birth,
        Child.uuid,
        Child.family_uuid,
        Child.class1_uuid,
        Child.class2_uuid,
        Child.class3_uuid
    ).filter_by(uuid=uuid).with_session(session).first()


def find_waiting_children():
    _children_from_database = Query(Child).with_entities(
        Child.uuid,
        Child.firstname,
        Child.lastname,
        Child.date_of_birth,
        Child.date_of_registration,
        Child.class1_uuid
    ).filter(Child.class1_uuid == "None").order_by(
        Child.date_of_birth.asc(),
        Child.date_of_registration.asc()
    ).with_session(session).all()

    _w_children = []
    for c in _children_from_database:
        _starts_in_year = calculate_startyear(c[3])
        _w_children.append({
            'uuid': c[0],
            'firstname': c[1],
            'lastname': c[2],
            'date_of_birth': c[3],
            'class1_uuid': c[5],
            '_starts_in': _starts_in_year,
            'date_of_registration': rearrange_date.to_list(c[4])[0]
        })
    return _w_children


def stringdate():
    _today = date.today()
    _date_list = str(_today).split('-')
    # build string in format 01-01-2000
    _date_string = _date_list[1] + "-" + _date_list[2] + "-" + _date_list[0]
    return _date_string


def get_family():
    session.close()
    _families = Query([Family.uuid, Family.parent1_uuid, Family.parent2_uuid]).with_session(session).all()
    _fam_with_parents = []
    for _fam in _families:
        parent1 = find_parents(_fam['parent1_uuid'])
        parent2 = find_parents(_fam['parent2_uuid'])
        print(parent2)
        if parent2 is None:
            _fam_with_parents.append({
                'family_uuid': _fam['uuid'],
                'parent1': parent1['firstname']+' '+parent1['lastname'],
                'parent2': ''
            })
        else:
            _fam_with_parents.append({
                'family_uuid': _fam['uuid'],
                'parent1': parent1['firstname']+' '+parent1['lastname'],
                'parent2': parent2['firstname']+' '+parent2['lastname']
            })
    session.close()
    return _fam_with_parents


def get_appropriate_class(uuid, years):
    available_groups = query_class_by_year(calculate_startyear(Query(Child.date_of_birth).filter_by(uuid=uuid).with_session(session)[0][0], years=years)[3:7])
    groups_list = []
    for i in available_groups:
        groups_list.append((i["class_uuid"], i["class_name"]))
    groups_list.insert(0, (None, 'Kies een klas'))
    print(groups_list)
    return groups_list


def assign_child_to_class():
    pass
#TODO
# vanuit index direct een klas toewijzen, via een popup?


def find_active_class(uuid):
    current_date = date.today()
    class1 = Query(Class).with_entities(Class.uuid, Class.start_date, Class.end_date).filter_by(uuid=uuid)
    print(class1)
    if  class1['start_date'] >= current_date <= class1['end_date']:
        print('Active')
        return True
    else:
        return False


from sqlalchemy import Column, Integer, String, Boolean
from sqlalchemy import create_engine, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import backref, relationship, Session
from src.modules.config import load_config
from os.path import exists


config = load_config()
def sessionSetup():
    engine = create_engine(
        f'sqlite:///{config["database"]["path"] + config["database"]["name"]}' + '?check_same_thread=False',
        echo=False,
    )
    return Session(bind=engine, autoflush=True)


def __init__():
    session = sessionSetup()


Base = declarative_base()


class Audit(Base):
    __tablename__ = 'tbl_audit'
    id = Column(Integer, primary_key=True)
    date = Column(Integer)
    action = Column(String)
    uuid = Column(String)

    def __init__(self, aid, date, action, uuid):
        super().__init__()
        self.aid = aid
        self.date = date
        self.action = action
        self.uuid = uuid


class Teacher(Base):
    __tablename__ = 'tbl_teachers'
    id = Column(Integer, primary_key=True)
    firstname = Column(String)
    lastname = Column(String)
    uuid = Column(String)

    def __init__(self, tid, firstname, lastname, uuid):
        super().__init__()
        self.tid = tid
        self.firstname = firstname
        self.lastname = lastname
        self.uuid = uuid


class Class(Base):
    __tablename__ = 'tbl_classrooms'
    id = Column(Integer, primary_key=True)
    class_name = Column(String)
    teacher_uuid = Column(String, ForeignKey('tbl_teachers.uuid'))
    teacher = relationship("Teacher", foreign_keys=[teacher_uuid])
    start_date = Column(Integer)
    end_date = Column(Integer)
    uuid = Column(String)

    def __init__(self, class_id, class_name, teacher_uuid, start_date, end_date, uuid):
        super().__init__()
        self.class_id = class_id
        self.class_name = class_name
        self.teacher_uuid = teacher_uuid
        self.start_date = start_date
        self.end_date = end_date
        self.uuid = uuid


class Parent(Base):
    __tablename__ = 'tbl_parents'
    id = Column(Integer, primary_key=True)
    firstname = Column(String)
    lastname = Column(String)
    address = Column(String)
    zipcode = Column(String)
    city = Column(String)
    email = Column(String)
    phone = Column(String)
    uuid = Column(String)

    def __init__(self, oid, firstname, lastname, address, zipcode, city, email, phone, uuid):
        super().__init__()
        self.oid = oid
        self.firstname = firstname
        self.lastname = lastname
        self.address = address
        self.zipcode = zipcode
        self.city = city
        self.email = email
        self.phone = phone
        self.uuid = uuid


class Family(Base):
    __tablename__ = 'tbl_families'
    id = Column(Integer, primary_key=True)
    parent1_uuid = Column(String, ForeignKey('tbl_parents.uuid'))
    parent1 = relationship("Parent", foreign_keys=[parent1_uuid])
    parent2_uuid = Column(String, ForeignKey('tbl_parents.uuid'))
    parent2 = relationship("Parent", foreign_keys=[parent2_uuid])
    divorced = Column(Integer)
    uuid = Column(String)

    def __init__(self, oid, parent1_uuid, parent2_uuid, divorced, uuid):
        super().__init__()
        self.oid = oid
        self.parent1_uuid = parent1_uuid
        self.parent2_uuid = parent2_uuid
        self.divorced = divorced
        self.uuid = uuid


class Child(Base):
    __tablename__ = 'tbl_children'
    id = Column(Integer, primary_key=True)
    firstname = Column(String)
    lastname = Column(String)
    date_of_registration = Column(Integer)
    family_uuid = Column(String, ForeignKey('tbl_families.uuid'))
    family = relationship("Family", foreign_keys=[family_uuid])
    date_of_birth = Column(Integer)
    class1_uuid = Column(String, ForeignKey('tbl_classrooms.uuid'))
    classroom1 = relationship("Class", foreign_keys=[class1_uuid])
    class2_uuid = Column(String, ForeignKey('tbl_classrooms.uuid'))
    classroom2 = relationship("Class", foreign_keys=[class2_uuid])
    class3_uuid = Column(String, ForeignKey('tbl_classrooms.uuid'))
    classroom3 = relationship("Class", foreign_keys=[class3_uuid])
    uuid = Column(String)

    def __init__(self, kid, firstname, lastname, date_of_registration, family_uuid, date_of_birth, class1_uuid,
                 class2_uuid, class3_uuid, uuid):
        super().__init__()
        self.kid = kid
        self.firstname = firstname
        self.lastname = lastname
        self.date_of_registration = date_of_registration
        self.family_uuid = family_uuid
        self.date_of_birth = date_of_birth
        self.class1_uuid = class1_uuid
        self.class2_uuid = class2_uuid
        self.class3_uuid = class3_uuid
        self.uuid = uuid


def create_database(databasefile):
    engine = create_engine('sqlite:////' + databasefile, echo=True)
    Base.metadata.create_all(engine)


def test_database(databasefile):
    # Check if database exists. If not, create it.
    file_exists = exists(databasefile)
    if not file_exists:
        print(f'Database aanmaken..')
        create_database(databasefile=databasefile)

if __name__ == '__main__':
#     .run(debug=True)
    create_database(databasefile='/home/yvesvanelk/Git/werfklas/werfklas.db')
# def __init__(self):

# ------ lisnters



from src.modules.common import find_class, find_teacher, stringdate


def get_classroom(uuid):
    # provision classroom based upon class id per child.
    # find classroom
    if uuid == "None":
        return None
    classrooms = find_class(uuid)
    # find teacher
    teachers = find_teacher(classrooms['teacher_uuid'])

    _classroom = ({
        'name': classrooms['class_name'],
        'uuid': classrooms['uuid'],
        'start_date': classrooms['start_date'],
        'end_date': classrooms['end_date'],
        'teacher_uuid': classrooms['teacher_uuid'],
        'teacher_firstname': teachers['firstname'],
        'teacher_lastname': teachers['lastname'],
        'active': (classrooms['start_date'] >= stringdate() <= classrooms['end_date'])
    })
    return _classroom
